/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-    */
/* ex: set filetype=cpp softtabstop=4 shiftwidth=4 tabstop=4 cindent expandtab: */

/*
  $Id: mtsMedtronicStealthlinkExampleComponent.h 3394 2012-01-26 20:24:46Z adeguet1 $

  Author(s):  Peter Kazanzides, Anton Deguet
  Created on: 2011-07-14

  (C) Copyright 2011-2012 Johns Hopkins University (JHU), All Rights Reserved.

--- begin cisst license - do not edit ---

This software is provided "as is" under an open source license, with
no warranty.  The complete license can be found in license.txt and
http://www.cisst.org/cisst/license.txt.

--- end cisst license ---
*/

#include <cisstMultiTask/mtsTaskPeriodic.h>

#ifndef _mtsMedtronicStealthlinkExampleComponent_h
#define _mtsMedtronicStealthlinkExampleComponent_h

#include <vtkSmartPointer.h>
#include <vtkPolyDataMapper.h>
#include <vtkActor.h>
#include <vtkSimplePointsReader.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkRenderer.h>
#include <vtkProperty.h>
#include <vtkPolyData.h>
#include <vtkGenericDataObjectReader.h>
#include <vtkLineSource.h>
#include <vtkModifiedBSPTree.h>
#include <vtkPoints.h>
#include <vtkLandmarkTransform.h>
#include <vtkTransformPolyDataFilter.h>
#include <vtkIterativeClosestPointTransform.h>

// define to not use any medtronic specific payloads (defined in mtsMedtronicStealthlinkTypes)
// #define SAW_MEDTRONIC_WITHOUT_STEALTHLINK_TYPES

class mtsMedtronicStealthlinkExampleComponent: public mtsTaskPeriodic
{
public:

    struct ToolStruct {
        mtsFunctionRead GetMarkerCartesian;
		mtsFunctionRead GetHindCartesian;
        mtsFunctionRead GetPositionCartesian;
    };

    struct CollectorStateStruct {
        mtsFunctionVoid StartCollection;
        mtsFunctionVoid StopCollection;
    };

    struct TestComponentStruct{
        mtsFunctionVoid StateTableAdvance;
    };

    void BatchReadyHandler(const mtsStateTable::IndexRange & range) {
        this->BatchReadyEventCounter++;
        this->LastRange = range;
    }

    void CollectionStartedHandler(void) {
        this->CollectionRunning = true;
    }

    void CollectionStoppedHandler(const mtsUInt & samplesCollected) {
        this->CollectionRunning = false;
        this->SamplesCollected = samplesCollected;
    }

    mtsMedtronicStealthlinkExampleComponent(const std::string & name,
                                            double periodInSeconds);

    mtsMedtronicStealthlinkExampleComponent(const std::string & name,
                                            double periodInSeconds,
                                            bool enableStateCollectionInterface);

    ~mtsMedtronicStealthlinkExampleComponent() {}

    void Run(void);
    void GetTool(void);

    ToolStruct Pointer;
    ToolStruct Frame;

    unsigned int BatchReadyEventCounter; // counter for range events from state table
    mtsStateTable::IndexRange LastRange;
    bool CollectionRunning;
    unsigned int SamplesCollected;
    CollectorStateStruct CollectorState;
    TestComponentStruct TestComponent;

protected:
    void AddStealthlinkInterface();
	void AddKeyboardInterface();
	void ReadFiducials();
	void InitVariables();

    void AddStateCollectionInterface();

    void AddToolInterface(const std::string & toolName,
                          ToolStruct & functionSet);

	void ReadArtery();

	bool IsOperating;
	bool IsRegistered;
	bool IsCollecting;
	bool IsCorrected;
	bool ShotLaser;

	int NoOfRPointsCollected;
	int NoOfIntersects;
	int NoOfIntersectFailures;

	void SwitchPreOp(void);
	void SwitchOp(void);
	void Register(void);
	void Collect(void);
	void ShootLaser(void);
	void CorrectRegistration(void);

	double diag;
	double point[3];

	double origError;
	double newError;

	vtkSmartPointer<vtkSimplePointsReader> fidReader;
	vtkSmartPointer<vtkPoints> ctFidPoints;
	vtkSmartPointer<vtkPoints> frameFidPoints;
	vtkSmartPointer<vtkGenericDataObjectReader> surfReader;
	vtkSmartPointer<vtkPoints> frameIntersectPoints;
	vtkSmartPointer<vtkTransform> translateFrame;
	vtkSmartPointer<vtkLandmarkTransform> registerTrue;
	vtkPolyData *ctArtery;
	vtkSmartPointer<vtkTransformPolyDataFilter> transformTrue;
	vtkSmartPointer<vtkTransformPolyDataFilter> introError;
	vtkSmartPointer<vtkTransformPolyDataFilter> transformTrueFid;
	vtkSmartPointer<vtkLandmarkTransform> registerError;
	vtkSmartPointer<vtkTransformPolyDataFilter> transformError;
	vtkSmartPointer<vtkTransformPolyDataFilter> transformErrorFid;
	vtkSmartPointer<vtkModifiedBSPTree> bspTree;
	vtkIdType noIntersect;
	vtkSmartPointer<vtkIterativeClosestPointTransform> icp;
	vtkSmartPointer<vtkTransformPolyDataFilter> transformCorrected;
	vtkSmartPointer<vtkTransformPolyDataFilter> transformCorrectedFid;
	vtkSmartPointer<vtkLandmarkTransform> registerCorrected;
};

#endif // _mtsMedtronicStealthlinkExampleComponent_h
